@@ -84,7 +84,7 @@ def train_from_folder(
     evaluate_every = 1000,
     generate = False,
     generate_interpolation = False,
-    hamburger_res_layers = [32],
+    attn_res_layers = [32],
     sle_spatial = False,
     disc_output_size = 1,
     interpolation_num_steps = 100,
@@ -104,7 +104,7 @@ def train_from_folder(
         models_dir = models_dir,
         batch_size = batch_size,
         gradient_accumulate_every = gradient_accumulate_every,
-        hamburger_res_layers = cast_list(hamburger_res_layers),
+        attn_res_layers = cast_list(attn_res_layers),
         sle_spatial = sle_spatial,
         disc_output_size = disc_output_size,
         image_size = image_size,
